@@ -12,7 +12,7 @@
 ****************************************************************************/
 /* Particle emitters
  *
- * $Id: emittermodule.c 102 2008-09-01 03:23:09Z casey.duncan $ */
+ * $Id: emittermodule.c 180 2009-03-15 14:36:12Z casey.duncan $ */
 
 #include <Python.h>
 #include <structmember.h>
@@ -83,7 +83,7 @@ Emitter_parse_kwargs(StaticEmitterObject *self,
 					"StaticEmitter: expected float value for rate");
 				return 0;
 			}
-			self->rate = PyFloat_AS_DOUBLE(value);
+			self->rate = (float)PyFloat_AS_DOUBLE(value);
 			Py_DECREF(value);
 			PyDict_DelItemString(kwargs, "rate");
 		}
@@ -97,7 +97,7 @@ Emitter_parse_kwargs(StaticEmitterObject *self,
 					"StaticEmitter: expected float value for time_to_live");
 				return 0;
 			}
-			self->time_to_live = PyFloat_AS_DOUBLE(value);
+			self->time_to_live = (float)PyFloat_AS_DOUBLE(value);
 			Py_DECREF(value);
 			PyDict_DelItemString(kwargs, "time_to_live");
 		}
@@ -249,8 +249,11 @@ Vec3_fill(Vec3 * __restrict__ vec, PyObject *domain, PyObject *discrete_seq,
 		v = PyObject_CallMethod(domain, "generate", NULL);
 		if (v == NULL)
 			return 0;
-		if (!Vec3_FromSequence(vec, v))
+		if (!Vec3_FromSequence(vec, v)) {
+			Py_DECREF(v);
 			return 0;
+		}
+		Py_DECREF(v);
 	} else if (discrete_seq != NULL) {
 		v = PySequence_Fast_GET_ITEM(discrete_seq,
 				(Py_ssize_t)(PySequence_Fast_GET_SIZE(discrete_seq) * rand_uni()));
@@ -275,8 +278,11 @@ Color_fill(Color * __restrict__ color, PyObject *domain, PyObject *discrete_seq,
 		v = PyObject_CallMethod(domain, "generate", NULL);
 		if (v == NULL)
 			return 0;
-		if (!Color_FromSequence(color, v))
+		if (!Color_FromSequence(color, v)) {
+			Py_DECREF(v);
 			return 0;
+		}
+		Py_DECREF(v);
 	} else if (discrete_seq != NULL) {
 		v = PySequence_Fast_GET_ITEM(discrete_seq,
 				(Py_ssize_t)(PySequence_Fast_GET_SIZE(discrete_seq) * rand_uni()));
@@ -310,7 +316,7 @@ Float_fill(float * f, PyObject *domain, PyObject *discrete_seq, float tmpl)
 	if (v != NULL) {
 		pyfloat = PyNumber_Float(v);
 		if (pyfloat != NULL) {
-			*f = PyFloat_AS_DOUBLE(pyfloat);
+			*f = (float)PyFloat_AS_DOUBLE(pyfloat);
 			result = 1;
 		}
 	} else {
@@ -436,7 +442,7 @@ Emitter_emit(StaticEmitterObject *self, PyObject *args)
 	GroupObject *pgroup;
 	long pindex;
 
-	if (!PyArg_ParseTuple(args, "iO:__init__", &count, &pgroup))
+	if (!PyArg_ParseTuple(args, "iO:emit", &count, &pgroup))
 		return NULL;
 	
 	if (!GroupObject_Check(pgroup))
@@ -463,9 +469,9 @@ Emitter_emit(StaticEmitterObject *self, PyObject *args)
 }
 
 static struct PyMemberDef StaticEmitter_members[] = {
-    {"rate", T_FLOAT, offsetof(StaticEmitterObject, rate), RESTRICTED,
+    {"rate", T_FLOAT, offsetof(StaticEmitterObject, rate), 0,
         "Rate of particle emission per unit time"},
-    {"time_to_live", T_FLOAT, offsetof(StaticEmitterObject, time_to_live), RESTRICTED,
+    {"time_to_live", T_FLOAT, offsetof(StaticEmitterObject, time_to_live), 0,
         "Time remaining before emitter is removed from the group (-1 to disable)"},
 	{NULL}
 };
@@ -695,7 +701,7 @@ PerParticleEmitter_call(PerParticleEmitterObject *self, PyObject *args)
 						return NULL;
 					remaining--;
 				}
-				total += count;
+				total += (long)count;
 			}
 			p++;
 		}
@@ -755,11 +761,11 @@ PerParticleEmitter_emit(PerParticleEmitterObject *self, PyObject *args)
 }
 
 static struct PyMemberDef PerParticleEmitter_members[] = {
-    {"rate", T_FLOAT, offsetof(PerParticleEmitterObject, rate), RESTRICTED,
+    {"rate", T_FLOAT, offsetof(PerParticleEmitterObject, rate), 0,
         "Number of particles per particle in the source group emitted per unit time"},
-    {"time_to_live", T_FLOAT, offsetof(PerParticleEmitterObject, time_to_live), RESTRICTED,
+    {"time_to_live", T_FLOAT, offsetof(PerParticleEmitterObject, time_to_live), 0,
         "Time remaining before emitter is removed from the group (-1 to disable)"},
-    {"source_group", T_OBJECT, offsetof(PerParticleEmitterObject, source_group), RESTRICTED,
+    {"source_group", T_OBJECT, offsetof(PerParticleEmitterObject, source_group), 0,
         "Source particle group containing template particles"},
 	{NULL}
 };
@@ -895,5 +901,5 @@ initemitter(void)
 	Py_INCREF(&PerParticleEmitter_Type);
 	PyModule_AddObject(m, "PerParticleEmitter", (PyObject *)&PerParticleEmitter_Type);
 
-	rand_seed(time(NULL));
+	rand_seed((unsigned long)time(NULL));
 }
