@@ -1,6 +1,6 @@
 /* Vector and color support
  *
- * $Id: vector.h 102 2008-09-01 03:23:09Z casey.duncan $ 
+ * $Id: vector.h 178 2009-03-15 06:11:32Z casey.duncan $ 
  */
 
 #include <Python.h>
@@ -9,6 +9,11 @@
 #ifndef _VECTOR_H_
 #define _VECTOR_H_
 
+#ifdef _MSC_VER
+#define inline
+#define __restrict__
+#endif
+
 /* 3D vector */
 typedef struct {
 	float x; float y; float z;
@@ -24,19 +29,17 @@ typedef struct {
 
 #define EPSILON 0.00001f
 
-typedef union {
-	int i;
-	float f;
-} floatint;
-
 /* The illustrious fast inverse sqrt of Quake 3 fame 
  * This algorithm is very fast, but at the cost of accuracy
  */
 static inline float InvSqrt (float x) {
     float xhalf = 0.5f*x;
-	int i = ((floatint)x).i;
+    int i = *(int*)&x;
     i = 0x5f3759df - (i>>1);
-	x = ((floatint)i).f;
+    x = *(float*)&i;
+	/* run two iterations of newton's method for 
+	   increased accuracy */
+    x = x*(1.5f - xhalf*x*x);
     x = x*(1.5f - xhalf*x*x);
     return x;
 }
@@ -108,22 +111,53 @@ static inline float InvSqrt (float x) {
 	(result)->z = -(v)->z
 
 #define Vec3_cross(result, v0, v1) \
-	(result)->x = ((v0)->y * (v1)->z) - ((v0)->z * (v1)->y) \
-	(result)->y = ((v0)->z * (v1)->x) - ((v0)->x * (v1)->z) \
-	(result)->x = ((v0)->x * (v1)->y) - ((v0)->y * (v1)->x)
+	(result)->x = ((v0)->y * (v1)->z) - ((v0)->z * (v1)->y); \
+	(result)->y = ((v0)->z * (v1)->x) - ((v0)->x * (v1)->z); \
+	(result)->z = ((v0)->x * (v1)->y) - ((v0)->y * (v1)->x)
 
 #define Vec3_len_sq(v) ((v)->x*(v)->x + (v)->y*(v)->y + (v)->z*(v)->z)
 
-#define Vec3_normalize(result, v) \
-	Vec3_scalar_mul(result, v, InvSqrt(Vec3_len_sq(v)))
+/* Fast vector normalize at the expense of accuracy
+   Return true if the vector was non-zero and could be normalized
+*/
+static inline int Vec3_normalize_fast(Vec3 *result, Vec3 *v)
+{
+	float len;
+	len = Vec3_len_sq(v);
+	Vec3_scalar_mul(result, v, InvSqrt(len));
+	return len > EPSILON;
+}
+
+/* Slower but more precise normalize 
+   Return true if the vector was non-zero and could be normalized
+*/
+static inline int Vec3_normalize(Vec3 *result, Vec3 *v)
+{
+	float len;
+	len = Vec3_len_sq(v);
+	if (len > EPSILON) {
+		len = 1.0f / sqrtf(len);
+		Vec3_scalar_mul(result, v, len);
+		return 1;
+	} else {
+		/* Maybe we should just clamp to zero here
+		   But I erred on the side of less data loss
+		   Callers concerned about this should pre or post-check
+		   the vector length.
+		*/
+		result->x = v->x;
+		result->y = v->y;
+		result->z = v->z;
+		return 0;
+	}
+}
 
 /* Populate a color from a Python sequence of 3 or 4 numbers */
 static inline int Color_FromSequence(Color *dest, PyObject *sequence);
 
-static inline float Vec3_len(Vec3 *v) {
-	float len;
-	len = Vec3_len_sq(v);
-	return len ? (1.0f / InvSqrt(len)) : 0.0f;
+static inline float Vec3_len(Vec3 *v) 
+{
+	return sqrtf(Vec3_len_sq(v));
 }
 
 static inline void Vec3_lerp(Vec3 *result, float t, const Vec3 *v0, const Vec3 *v1)
@@ -173,16 +207,56 @@ static inline void Vec3_copy(Vec3 * __restrict__ dest, Vec3 * __restrict__ src)
 	dest->z = src->z;
 }
 
+/* Given an arbitrary, non-zero LOS vector, create 3 orthogonal normalized vectors
+   cooresponding to the out, up and right axis of the cooresponding 3x3 rotation
+   matrix that may be used to tranform points along LOS. The up and right vectors
+   are arbitrarily aligned for best results.
+
+   Return true on success.
+*/
+static inline int
+Vec3_create_rot_vectors(Vec3 * __restrict__ los, Vec3 * __restrict__ out,
+	Vec3 * __restrict__ up, Vec3 * __restrict__ right)
+{
+	Vec3 world_up, tmp;
+
+	if (Vec3_normalize(out, los)) {
+		world_up.x = 0.0f; world_up.y = 0.0f; world_up.z = 1.0f;
+		Vec3_scalar_mul(&tmp, out, Vec3_dot(&world_up, out));
+		Vec3_sub(up, &world_up, &tmp);
+		if (Vec3_len_sq(up) < EPSILON) {
+			/* Try another world up axis */
+			world_up.x = 0.0f; world_up.y = 1.0f; world_up.z = 0.0f;
+			Vec3_scalar_mul(&tmp, out, Vec3_dot(&world_up, out));
+			Vec3_sub(up, &world_up, &tmp);
+			if (Vec3_len_sq(up) < EPSILON) {
+				/* Try yet another world up axis */
+				world_up.x = 1.0f; world_up.y = 0.0f; world_up.z = 0.0f;
+				Vec3_scalar_mul(&tmp, out, Vec3_dot(&world_up, out));
+				Vec3_sub(up, &world_up, &tmp);
+				if (Vec3_len_sq(up) < EPSILON) {
+					return 0;
+				}
+			}
+		}
+		Vec3_normalize(up, up);
+		Vec3_cross(right, up, out);
+		return 1;
+	}
+	return 0;
+}
 
 /* Return the closest point along a line segment to the given point */
-static inline void Vec3_closest_pt_to_line(Vec3 * __restrict__ dest, Vec3 * __restrict__ pt, 
+static inline void 
+Vec3_closest_pt_to_line(Vec3 * __restrict__ dest, Vec3 * __restrict__ pt, 
 	Vec3 * __restrict__ lstart, Vec3 * __restrict__ lend)
 {
 	Vec3 tp, lv;
+	float mag2;
 
 	Vec3_sub(&lv, lend, lstart);
 	Vec3_sub(&tp, pt, lstart);
-	const float mag2 = Vec3_len_sq(&lv);
+	mag2 = Vec3_len_sq(&lv);
 	if (mag2 > EPSILON) {
 		float t = Vec3_dot(&tp, &lv) / mag2;
 		t = clamp(t, 0.0f, 1.0f);
@@ -194,28 +268,8 @@ static inline void Vec3_closest_pt_to_line(Vec3 * __restrict__ dest, Vec3 * __re
 	}
 }
 
-/*
-static static inline void vmathV3Slerp( VmathVector3 *result, float t, const VmathVector3 *unitVec0, const VmathVector3 *unitVec1 )
-{
-    VmathVector3 tmpV3_0, tmpV3_1;
-    float recipSinAngle, scale0, scale1, cosAngle, angle;
-    cosAngle = vmathV3Dot( unitVec0, unitVec1 );
-    if ( cosAngle < _VECTORMATH_SLERP_TOL ) {
-        angle = acosf( cosAngle );
-        recipSinAngle = ( 1.0f / sinf( angle ) );
-        scale0 = ( sinf( ( ( 1.0f - t ) * angle ) ) * recipSinAngle );
-        scale1 = ( sinf( ( t * angle ) ) * recipSinAngle );
-    } else {
-        scale0 = ( 1.0f - t );
-        scale1 = t;
-    }
-    vmathV3ScalarMul( &tmpV3_0, unitVec0, scale0 );
-    vmathV3ScalarMul( &tmpV3_1, unitVec1, scale1 );
-    vmathV3Add( result, &tmpV3_0, &tmpV3_1 );
-}
-*/
-
-static inline void Color_copy(Color * __restrict__ dest, Color * __restrict__ src) 
+static inline void 
+Color_copy(Color * __restrict__ dest, Color * __restrict__ src) 
 {
 	dest->r = src->r;
 	dest->g = src->g;
@@ -223,5 +277,4 @@ static inline void Color_copy(Color * __restrict__ dest, Color * __restrict__ sr
 	dest->a = src->a;
 }
 
-
 #endif
